\name{nclust}
\alias{nclust}
\alias{mostvar}
\alias{mostvaridx}
\title{Fast Hierarchical Clustering}
\description{
Hierarchical cluster analysis, implemented using
the reciprocal nearest neighbor chain algorithm.
}
\usage{
nclust(data, link = c("ave","mcquitty","ward","single","complete","allpair"),
  branchflip = c("nephew","tightleft"), 
  center = c("col", "row","both","none"), impute=TRUE,
  distance = c("ward","euclidean"),
  height = c("loglevel","linkscore")
  )

mostvaridx(x,ntop)

mostvar(x,ntop)
}
\arguments{
  \item{data}{Data to be clustered. Currently, it can be a pairwise
  distance object of class \code{"dist"}, or a numeric matrix with
  items to be clustered as columns. For the latter, missing values
  are allowed.
  }
  \item{link}{Link function between clusters. These are a subset
  of those in \link{hclust} that satisfy the
  reducibility property and amenable to reciprocal nearest-neighbor chain
  algorithm.}
  \item{branchflip}{The default is flip the branches such that
  the nearest nephew is placed close to its uncle. \code{tightleft}
  puts the tighter branch on the left, identical to that of \link{hclust}}
  \item{center}{Mean centering of the data before the distance is
  computed. The default is column centering, which give one minus Pearson's
  correlation. If more complex centering is desired, e.g. including
  batch effects, the data matrix should be pre-centered before calling
  \code{nclust} with \code{center="none"}. Not applicable if
  the input is a pairwise distance object.}
  \item{impute}{Replace non-finite values by column means.}
  \item{distance}{Distance method. The default \code{"ward"} is one
  minus correlation. Note that \code{"ward"} here is unrelated
  to that in the option \code{"link"}. Not applicable for
  pairwise distance input.}
  \item{height}{The tree height. The option \code{"linkscore"} is
  the tradional way to put the internal node at the distance proportional
  to the link score. The default \code{"loglevel"} uses the logarithm
  of the level, with the root at level 1.}
  \item{x}{A data matrix}
  \item{ntop}{The requested number of \code{ntop} most variable columns}
}
\details{
When hierarchical clustering is performed using link functions that
satisfy the ``reducibility property'', then it is possible
to do the computation with quadratic instead of cubic complexity.
Furthermore, there is no need to pre-compute and store all
pairwise distances, which can be very large for large number
of items to be clustered. The pairwise distance still need
to be computed for each pair, but the tree can be constructed
on-the-fly using the reciprocal nearest-neighbor chain algorithm.
The results is identical to those obtained by brute-force
cubic-time algorithm, or the slightly faster versions
that uses priority queue (an $0(N^2 log N)$ algorithm).

\code{nclust} can be used in similar way to \link{hclust}, but
faster and with an option to directly input the data matrix
(as opposed to the pairwise distance) and a branch re-ordering
rule called ``nearest nephew'', which flip each fork of the
tree, such that the nephew that is closer in terms of the link
function is also visually closer. (Currently the link function
is not modifiable. It is
fixed to average linkage for dist object, and centroid for data matrices.)

\code{mostvaridx} returns column indices of the \code{ntop} most
variable columns of a matrix. \code{mostvar} return a subset
of the matrix with columns selected according to \code{mostvaridx}.
}
\value{
  A list with similar content to the output of \link{hclust}. 
  The tree plotting function \link{plclust} is applicable
  to this output.
  
  For a large tree, the functions
  \link{plot.nclust} and \link{dendrolines}
  are more appropriate and flexible.
}
\references{
Benzecri, J. P. (1982)
Construction d'une classification ascendante hierarchique
par la recherce en chaine des voisins reciproques.
Les Cahiers de l'Analyse des Donnees 7:209-218

Bruynooghe, M (1977)
Methodes nouvelles en classification automatique des donnees
taxinomiques nombreuses.
Statistique et Analyse des Donnees 3:24-42.

Gordon A.D. (1999) Classification. 2nd ed. Chapman and Hall/CRC.
}

\author{ Pratyaksha Wirapati }

\seealso{dendrolines, plot.nclust, hclust, plclust}

\examples{
  # hclust compatibility mode
  hc <- hclust(dist(USArrests),method="ave")
  nc1 <- nclust(dist(USArrests),link="ave",
    height="linkscore",branchflip="tightleft")
  plclust(hc)
  plclust(nc1)

  # nclust can take data matrix directly. Note that it
  # cluster the columns [!].

  nc2 <- nclust( t( as.matrix(USArrests)),link="ave", distance="euclidean",
    height="linkscore", branchflip="tightleft", center="none")
  plclust(nc2)

  # cluster the columns of a random array directly
  x <- array( rnorm(10*4000), dim=c(10,4000) )
  hx <- nclust(x, height="linkscore")
  plot.nclust(hx,branchtype="e")
  axis(1)
}
